* =======================================================================
*  Decomposed Capacitated Facility Location problem
*  coordinate (NLP) approach for plant location
*  nonlinear model (taken from Mat's paper) for plant cost
*  second scheme for decomposition: introducing copy vriables for q's
*  incremental subgradient method for solving sub-problems
* =======================================================================

sets

i      cities /1*10/
k      plant coordinates /x,y/
ii /1*10/
;

alias(j,i);alias(jj,j);

parameters

  cq(i)     demand at each city   /1  .45, 2  .50, 3 0.55, 4 0.65, 5 0.48,6  .32, 7  .81,8  0.28,9  0.66, 10  0.43/
  maxcap    maximum plant capacity               /2/
  mincap    a small number relative to the plant capacity for setting big-M /0.01/
  bsize     base size for the plant cost         /.50/
  bcost     cost for the plant with base size    /1./
  tcost     transportation cost per unit of distance /1./
  sfac      scale factor in the plant cost term  /0.72/
  l         domain size (assumed to be square)  /1.0/
  landa(i,j) lagrange multipliers for consistency constraint
  tlanda(i)
  alpha     step size in subgradient update
  quan(i,j) quantity trucked from the jth plant to ith city
  ploc(k,j) plant locations
  flower    lagrangian lower bound for the total cost
  fval(ii)  array of all lagrangian bounds
  comp(ii)


  epsilon  /0.01/
  il
  rnd
  upb
;

table  c(k,i)  city coordinates
    1      2      3     4      5     6     7      8      9      10
x  0.0   1.0    0.7   0.45   0.3    0.4   0.23   0.86   0.63    0.9
y  .80    .5    1.0   0.15   0.3    0.9   0.41   0.21   0.78    0.12

;

positive variables

p(k)    plant coordinates
q(i)    quantity trucked to each city
q2(i,j) copy variables

;

free variables
cost        total cost
copy;

p.lo(k)=0.0;     p.up(k)=1.0;
q.lo(i) = 0;     q.up(i) =cq(i);
q2.lo(i,j)=0;    q2.up(i,j) =cq(i);
cost.lo = -100;  cost.up = 50;



equations
obj1            objective function for nlps
obj2            objective function for lp
Capacity1       plant max capacity for nlps
Capacity2(j)    plant max capacity for lp
poff(k)         fixing the coordintes of non-existing plants in nlps
cityDemand(i)   satisfying city demands in lp
*b11,b12
;
* initial equations
obj1 .. cost=e=bcost*(sum(i,q(i))/bsize)**sfac+tcost*sum(i,q(i)*sqrt(sum(k,sqr(c(k,i)-p(k)))))-sum(i,tlanda(i)*q(i));

obj2 .. copy=e=sum(j,sum(i,landa(i,j)*q2(i,j)));
Capacity1 ..    sum(i, q(i)) =l= maxcap;
Capacity2(j) .. sum(i, q2(i,j)) =l= maxcap;
poff(k) ..     p(k)*mincap=l=sum(i,q(i));
cityDemand(i) .. sum(j,q2(i,j))=e=cq(i);

* branching equations

*b11 .. q2('2','5')=l=0.25;
*b12 .. q('2')=l=upb;

model snlp /obj1,capacity1,poff/;
model slp  /obj2,capacity2, citydemand/;

$onecho > baron.opt
maxtime 1200000
epsr 0.01
nlpsol 4
$offecho

slp.OptCR=0.001;

snlp.optfile=1;

option NLP =baron;
option LP=cplex;

landa(i,j)=0.0;
   comp(ii)=100;
*upb=cq('2');

loop (ii,
   flower=0.0;
   alpha=20/(0.1*(ord(ii)-1)+1);
* solving for a random plant using a fixed multiplier

*  if (round(rnd)>=11,
     solve slp using LP minimizing copy;
     flower=flower+copy.l;
     landa(i,j)=landa(i,j)+alpha*q2.l(i,j);
*  );

for (il=1 to 5,
   rnd=uniform(1,10);
   loop (j,
      if (ord(j)=round(rnd),
       display rnd;
       tlanda(i)=landa(i,j);
       q.l(i) =cq(i);
       solve snlp using NLP minimizing cost;
       flower=flower+cost.l;
       quan(i,j)=q.l(i);
       ploc(k,j)=p.l(k);
       landa(i,j)=landa(i,j)-alpha*quan(i,j);
       );
  );
 );

  fval(ii)=flower;

*  comp(ii)=smax(i,smax(j,abs(landa(i,j)*(q2.l(i,j)-quan(i,j)))));

  display ploc,quan, q2.l , landa,comp,fval;

*  if (comp(ii)<epsilon, abort "execution finished",fval);

);

